/*
 * anjuta.h Copyright (C) 2000  Kh. Naba Kumar Singh
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free 
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 59 
 * Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

#ifndef _ANJUTA_H_
#define _ANJUTA_H_

#include "gtkantext.h"
#include "toolbar.h"
#include "text_editor.h"
#include "messagebox.h"
#include "preferences.h"
#include "compiler_options.h"
#include "src_paths.h"
#include "find_replace.h"
#include "find_in_files.h"
#include "messages.h"
#include "project_dbase.h"
#include "breakpoints.h"
#include "anjuta_dirs.h"
#include "executer.h"
#include "configurer.h"
#include "utilities.h"
#include "main_menubar.h"

typedef struct _AnjutaAppGui AnjutaAppGui;
typedef struct _AnjutaApp AnjutaApp;
typedef struct _FileLineInfo FileLineInfo;

struct _AnjutaAppGui
{
  GtkWidget *window;
  GtkWidget *client_area;
  GtkWidget *the_client;
  GtkWidget *hpaned_client;
  GtkWidget *hpaned;
  GtkWidget *vpaned;
  GtkWidget *project_dbase_win_container;
  GtkWidget *mesg_win_container;
  GtkWidget *notebook;
  GtkWidget *appbar;

  Toolbar toolbar;
  MainMenuBar menubar;
};

struct _AnjutaApp
{
  AnjutaAppGui widgets;
  GtkWidget *fileselection;
  GtkWidget *save_as_fileselection;
  GList *text_editor_list;

  GtkAccelGroup* accel_group;
  FindAndReplace *find_replace;
  Messages *messages;
  ProjectDBase *project_dbase;
  Preferences *preferences;
  CompilerOptions *compiler_options;
  SrcPaths* src_paths;
  TextEditor *current_text_editor;
  AnjutaDirs *dirs;
  Executer *executer;
  Configurer *configurer;
  FindInFiles *find_in_files;
  TagsManager *tags_manager;

  GList *registered_windows;
  GList *registered_child_processes;
  GList *registered_child_processes_cb;
  GList *registered_child_processes_cb_data;
  GList *recent_files;
  GList *recent_projects;
  gint hpaned_width;
  gint vpaned_height;
  gint win_pos_x, win_pos_y, win_width, win_height;
  gboolean auto_gtk_update;

  GnomeAppProgressKey  progress_key;
  gdouble  progress_value;
  gdouble  progress_full_value;
  GnomeAppProgressCancelFunc progress_cancel_cb;
  gboolean in_progress;
  gint busy_count;
  GList *prog_list;
};

struct _FileLineInfo
{
   gchar *file;
   glong  line;
};

extern AnjutaApp *app;
extern int arg_c;
extern char** arg_v;

void create_anjuta_gui (AnjutaApp * appl);

void anjuta_new (void);

void anjuta_show (void);

TextEditor* anjuta_append_text_editor (gchar * filename);

void anjuta_remove_current_text_editor (void);

TextEditor *anjuta_get_current_text_editor (void);

GtkAnText *anjuta_get_current_text (void);

gchar* anjuta_get_current_selection(void);

void anjuta_goto_file_line (gchar * fname, guint lineno);

void anjuta_apply_preferences (void);

TextEditor *anjuta_get_notebook_text_editor (gint page_num);

void anjuta_save_settings (void);

gboolean anjuta_save_yourself (FILE * stream);

gboolean anjuta_load_yourself (FILE * stream);

void anjuta_clean_exit ();

void on_anjuta_app_realize (GtkWidget * w, gpointer data);

void on_anjuta_destroy (GtkWidget * w, gpointer data);

gint on_anjuta_delete (GtkWidget * w, GdkEvent *event, gpointer data);

void on_anjuta_exit_yes_clicked (GtkButton * b, gpointer data);

void anjuta_clean_exit (void);

void
on_anjuta_notebook_switch_page (GtkNotebook * notebook,
				GtkNotebookPage * page,
				gint page_num, gpointer user_data);
void anjuta_update_title (void);

void anjuta_open_pixmap(gchar* filename);

void anjuta_status (gchar * mesg);
void anjuta_warning (gchar * mesg);
void anjuta_error (gchar * mesg);
void anjuta_set_busy(void);
void anjuta_set_active(void);
gboolean anjuta_set_auto_gtk_update(gboolean auto_flag);

gboolean
on_anjuta_window_focus_in_event (GtkWidget * w, GdkEventFocus * e,
				 gpointer d);

void on_open_filesel_ok_clicked (GtkButton * button, gpointer data);

void on_open_filesel_cancel_clicked (GtkButton * button, gpointer data);

void on_save_as_filesel_ok_clicked (GtkButton * button, gpointer data);

void on_save_as_filesel_cancel_clicked (GtkButton * button, gpointer data);

void
on_save_as_overwrite_yes_clicked (GtkButton * button, gpointer user_data);

void
on_prj_list_undock_button_clicked (GtkButton * button, gpointer user_data);

void on_prj_list_hide_button_clicked (GtkButton * button, gpointer user_data);

void on_mesg_win_hide_button_clicked (GtkButton * button, gpointer user_data);

void
on_mesg_win_undock_button_clicked (GtkButton * button, gpointer user_data);

void
on_recent_files_menu_item_activate (GtkMenuItem *item, gchar * data);

void
on_recent_projects_menu_item_activate (GtkMenuItem *item, gchar * data);

void
main_menu_unref(void);

gchar*
anjuta_get_full_filename(gchar* fn);

gdouble
on_anjuta_progress_cb(gpointer data);

void
on_anjuta_progress_cancel(gpointer data);

void
anjuta_done_progress(gchar* titile);

gboolean
anjuta_init_progress(gchar* description, gdouble full_value,
	GnomeAppProgressCancelFunc progress_cancel_cb,
	gpointer data);

void
anjuta_set_progress(gdouble value);

void
anjuta_grab_text_focus(gint page);

void
anjuta_register_window(GtkWidget* win);

void
anjuta_unregister_window(GtkWidget* win);

void
anjuta_register_child_process(pid_t pid,
		void (*callback)(int status, gpointer d), gpointer data);
void
anjuta_unregister_child_process(pid_t pid);

void
anjuta_not_implemented(gchar* file, guint line);

void
anjuta_find_missings (gboolean show);

gboolean
anjuta_is_installed (gchar * prog, gboolean show);

#endif
